#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
import cherrypy
from woost.controllers import BaseCMSController


class IssuuDocumentController(BaseCMSController):

    def __call__(self, *args, **kwargs):
        uri = self.context["publishable"].get_issuu_uri()
        raise cherrypy.HTTPRedirect(uri)





#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from woost.models import Item, Configuration

_get_image_uri = Item.get_image_uri

def get_image_uri(
    self,
    image_factory = None,
    parameters = None,
    encode = True,
    include_extension = True,
    host = None
):
    if host in (None, "!", "?"):
        host = Configuration.instance.get_setting("external_files_host")

    return _get_image_uri(
        self,
        image_factory = image_factory,
        parameters = parameters,
        encode = encode,
        include_extension = include_extension,
        host = host
    )

Item.get_image_uri = get_image_uri



#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cocktail import schema
from cocktail.controllers import Location, make_uri
from woost.models.publishable import Publishable
from woost.models.controller import Controller
from woost.models.website import Website


class URI(Publishable):

    instantiable = True
    type_group = "resource"

    groups_order = ["content"]

    members_order = [
        "title",
        "uri",
        "language_specific_uri"
    ]

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(qname = "woost.uri_controller")
    )

    title = schema.String(
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True,
        member_group = "content"
    )

    uri = schema.String(
        indexed = True,
        member_group = "content"
    )

    language_specific_uri = schema.String(
        translated = True,
        member_group = "content"
    )

    def get_uri(self,
        path = None,
        parameters = None,
        language = None,
        host = None,
        encode = True):

        uri = self.language_specific_uri or self.uri

        if uri is not None:

            if path:
                uri = make_uri(uri, *path)

            if parameters:
                uri = make_uri(uri, **parameters)

            uri = self._fix_uri(uri, host, encode)

        return uri

    def is_internal_content(self, language = None):

        uri = self.get_uri(host = "!", language = language)
        location = Location(uri)

        if not location.host:
            return True

        return any(
            location.host in website.hosts
            for website in Website.select()
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from __future__ import with_statement
from types import GeneratorType
from threading import Lock
from time import time, mktime
from datetime import datetime
from hashlib import md5
import cherrypy
from cherrypy.lib import cptools, http
from cocktail.caching import CacheKeyError
from woost import app
from woost.controllers import BaseCMSController


class PublishableController(BaseCMSController):
    """Base controller for all publishable items (documents, files, etc)."""

    cache_enabled = True

    cached_headers = (
        "Content-Type",
        "Content-Length",
        "Content-Disposition",
        "Content-Encoding",
        "Last-Modified",
        "ETag"
    )

    def __call__(self, **kwargs):
        content = self._apply_cache(**kwargs)

        if content is None:
            content = self._produce_content(**kwargs)

        return content

    def _apply_cache(self, **kwargs):

        publishable = self.context["publishable"]
        request = cherrypy.request
        response = cherrypy.response

        if not (
            self.cache_enabled
            and publishable.cacheable
            and request.method == "GET"
            and "original_publishable" not in self.context
        ):
            return None

        caching_context = {
            "request": request,
            "controller": self
        }

        policy = publishable.get_effective_caching_policy(
            **caching_context
        )

        if policy is None or not policy.cache_enabled:
            return None

        # Find the unique cache identifier for the requested content
        cache_key = repr(policy.get_content_cache_key(
            publishable,
            **caching_context
        ))

        # Server side cache
        if policy.server_side_cache:

            try:
                cached_response = app.cache.retrieve(cache_key)

            except CacheKeyError:
                content = self.__produce_response(**kwargs)
                view = self.view
                expiration = policy.get_content_expiration(
                    publishable,
                    base = view and view.cache_expiration,
                    **caching_context
                )
                tags = policy.get_content_tags(
                    publishable,
                    base = view and view.cache_tags,
                    **caching_context
                )

                if isinstance(content, GeneratorType):
                    content = "".join(content)

                # Collect headers that should be included in the cache
                headers = {}
                response_headers = cherrypy.response.headers

                for header_name in self.cached_headers:
                    header_value = response_headers.get(header_name)
                    if header_value:
                        headers[header_name] = header_value

                # Store the response in the cache
                app.cache.store(
                    cache_key,
                    (headers, content),
                    expiration = expiration,
                    tags = tags
                )
            else:
                headers, content = cached_response
                cherrypy.response.headers.update(headers)
        else:
            content = self.__produce_response(**kwargs)

        # Client side cache
        cptools.validate_etags()
        return content

    def __produce_response(self, **kwargs):
        content = self._produce_content(**kwargs)

        if isinstance(content, GeneratorType):
            content_bytes = "".join(
                chunk.encode("utf-8")
                    if isinstance(chunk, unicode)
                    else chunk
                for chunk in content
            )
        elif isinstance(content, unicode):
            content_bytes = content.encode("utf-8")
        else:
            content_bytes = content

        cherrypy.response.headers["ETag"] = md5(content_bytes).hexdigest()
        return content_bytes

    def _produce_content(self, **kwargs):
        return BaseCMSController.__call__(self, **kwargs)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# Publishable
#------------------------------------------------------------------------------
translations.define("Publishable.access_restriction",
    ca = u"Restricció d'accés",
    es = u"Restricción de acceso",
    en = u"Access restriction"
)

translations.define("Publishable.access_restriction=None",
    ca = u"Sense restricció",
    es = u"Sin restricción",
    en = u"Unrestricted"
)

# AccessRestriction
#------------------------------------------------------------------------------
translations.define("AccessRestriction",
    ca = u"Restricció d'accés",
    es = u"Restricción de acceso",
    en = u"Access restriction"
)

translations.define("AccessRestriction-plural",
    ca = u"Restriccions d'accés",
    es = u"Restricciones de acceso",
    en = u"Access restrictions"
)

translations.define("AccessRestriction.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("AccessRestriction.restricted_content",
    ca = u"Contingut restringit",
    es = u"Contenido restringido",
    en = u"Restricted content"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from PIL import Image
from cocktail.events import event_handler
from cocktail.iteration import first
from cocktail import schema
from woost.models.item import Item
from woost.models.rendering.renderer import Renderer

def _get_site_renderers():
    from woost.models import Configuration
    return Configuration.instance.renderers


class ImageFactory(Item):

    visible_from_root = False

    members_order = [
        "title",
        "identifier",
        "renderer",
        "default_format",
        "effects",
        "fallback",
        "applicable_to_blocks"
    ]

    title = schema.String(
        translated = True,
        descriptive = True
    )

    identifier = schema.String(
        unique = True,
        indexed = True,
        normalized_index = False
    )

    renderer = schema.Reference(
        required = True,
        type = Renderer,
        related_end = schema.Collection(),
        default = schema.DynamicDefault(
            lambda: first(_get_site_renderers())
        ),
        enumeration = lambda ctx: _get_site_renderers(),
        edit_control = "cocktail.html.DropdownSelector"
    )

    default_format = schema.String(
        enumeration = ["JPEG", "PNG", "GIF"],
        translatable_enumeration = False,
        edit_control = "cocktail.html.DropdownSelector"
    )

    effects = schema.Collection(
        items = "woost.models.rendering.ImageEffect",
        bidirectional = True,
        integral = True,
    )

    fallback = schema.Reference(
        type = "woost.models.rendering.ImageFactory",
        bidirectional = True
    )

    fallback_referers = schema.Collection(
        items = "woost.models.rendering.ImageFactory",
        bidirectional = True,
        visible = False,
        synchronizable = False
    )

    applicable_to_blocks = schema.Boolean(
        required = True,
        default = True,
        indexed = True
    )

    def render(self, item):

        image = self.renderer.render(item)

        if image is None:
            if self.fallback:
                image = self.fallback.render(item)
        else:
            if self.effects:
                if isinstance(image, basestring):
                    image = Image.open(image)

                for effect in self.effects:
                    image = effect.apply(image)

        return image

    cache_invalidators = frozenset((
        identifier,
        renderer,
        effects,
        fallback
    ))

    @event_handler
    def handle_changing(cls, e):
        from woost.models.rendering.request \
            import clear_image_cache_after_commit
        if e.source.is_inserted and e.member in cls.cache_invalidators:
            clear_image_cache_after_commit(factory = e.source)
            for referer in e.source.fallback_referers:
                clear_image_cache_after_commit(factory = referer)

    @event_handler
    def handle_related(cls, e):
        from woost.models.rendering.request \
            import clear_image_cache_after_commit
        if e.source.is_inserted and e.member in cls.cache_invalidators:
            clear_image_cache_after_commit(factory = e.source)
            for referer in e.source.fallback_referers:
                clear_image_cache_after_commit(factory = referer)

    @event_handler
    def handle_unrelated(cls, e):
        from woost.models.rendering.request \
            import clear_image_cache_after_commit
        if e.source.is_inserted and e.member in cls.cache_invalidators:
            clear_image_cache_after_commit(factory = e.source)
            for referer in e.source.fallback_referers:
                clear_image_cache_after_commit(factory = referer)

    @event_handler
    def handle_deleting(cls, e):
        from woost.models.rendering.request \
            import clear_image_cache_after_commit
        clear_image_cache_after_commit(factory = e.source)
        for referer in e.source.fallback_referers:
            clear_image_cache_after_commit(factory = referer)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from woost.models import Document
from tpv.paypal import PayPalPaymentGateway as Implementation
from woost.extensions.payments.paymentgateway import PaymentGateway


class PayPalPaymentGateway(PaymentGateway, Implementation):

    instantiable = True

    default_label = schema.DynamicDefault(
        lambda: translations("PayPalPaymentGateway.label default")
    )

    members_order = [
        "business",
        "payment_successful_page",
        "payment_failed_page"
    ]

    business = schema.String(
        required = True,
        shadows_attribute = True,
        text_search = False
    )

    payment_successful_page = schema.Reference(
        type = Document,
        related_end = schema.Reference()
    )

    payment_failed_page = schema.Reference(
        type = Document,
        related_end = schema.Reference()
    )

    def get_payment_successful_url(self, payment):
        if self.payment_successful_page:
            return self.payment_successful_page.get_uri(
                host = "!",
                parameters = {"payment_id": payment.id}
            )

    def get_payment_failed_url(self, payment):
        if self.payment_failed_page:
            return self.payment_failed_page.get_uri(
                host = "!",
                parameters = {"payment_id": payment.id}
            )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations


# VimeoVideo
#------------------------------------------------------------------------------
translations.define("VimeoVideo",
    ca = u"Vídeo de Vimeo",
    es = u"Video de Vimeo",
    en = u"Vimeo video"
)

translations.define("VimeoVideo-plural",
    ca = u"Vídeos de Vimeo",
    es = u"Videos de Vimeo",
    en = u"Vimeo videos"
)

translations.define("VimeoVideo.content",
    ca = u"Contingut",
    es = u"Contenido",
    en = u"Content"
)

translations.define("VimeoVideo.title",
    ca = u"Títol",
    es = u"Título",
    en = u"Title"
)

translations.define("VimeoVideo.video_id",
    ca = u"Identificador del vídeo",
    es = u"Identificador del video",
    en = u"Video ID"
)

# VimeoVideoRenderer
#------------------------------------------------------------------------------
translations.define("VimeoVideoRenderer",
    ca = u"Pintador de fotogrames pels vídeos de Vimeo",
    es = u"Pintador de fotogramas para los videos de Vimeo",
    en = u"Vimeo videos still renderer"
)

translations.define("VimeoVideoRenderer-plural",
    ca = u"Pintadors de fotogrames pels vídeos de Vimeo",
    es = u"Pintadores de fotogramas para los videos de Vimeo",
    en = u"Vimeo videos still renderers"
)

# VideoPlayerSettings
#------------------------------------------------------------------------------
translations.define("VideoPlayerSettings.vimeo",
    ca = u"Vimeo",
    es = u"Vimeo",
    en = u"Vimeo"
)

translations.define("VideoPlayerSettings.vimeo_loop",
    ca = u"Reproduïr continuament",
    es = u"Reproducir continuamente",
    en = u"Loop"
)

translations.define("VideoPlayerSettings.vimeo_allow_fullscreen",
    ca = u"Permetre pantalla completa",
    es = u"Permitir pantalla completa",
    en = u"Allow fullscreen"
)

translations.define("VideoPlayerSettings.vimeo_title",
    ca = u"Mostrar el títol del vídeo",
    es = u"Mostrar el título del video",
    en = u"Show the video's title"
)

translations.define("VideoPlayerSettings.vimeo_byline",
    ca = u"Mostrar informació de l'autor",
    es = u"Mostrar información del autor",
    en = u"Show the video's byline"
)

translations.define("VideoPlayerSettings.vimeo_portrait",
    ca = u"Mostrar imatge de l'autor",
    es = u"Mostrar imagen del autor",
    en = u"Show the author's portrait"
)

translations.define("VideoPlayerSettings.vimeo_color",
    ca = u"Color del reproductor",
    es = u"Color del reproductor",
    en = u"Player color"
)




#-*- coding: utf-8 -*-
"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
import cherrypy
from cocktail import schema
from cocktail.controllers import (
    Controller,
    FormProcessor,
    Form,
    request_property,
    context
)
from cocktail.controllers.location import Location
from woost import app
from woost.models import Item
from woost.controllers.backoffice.usereditnode import PasswordConfirmationError
from woost.extensions.signup.signupconfirmationcontroller import generate_confirmation_hash


class SignUpController(FormProcessor, Controller):

    is_transactional = True

    class SignUpForm(Form):

        @request_property
        def model(self):
            return self.controller.block.user_type

        @request_property
        def adapter(self):
            adapter = Form.adapter(self)
            adapter.implicit_copy = False
            adapter.copy(("email", "password"))
            return adapter

        @request_property
        def schema(self):
            adapted_schema = Form.schema(self)

            # Set schema name in order to keep always the same value
            # although change value of form_model member
            adapted_schema.name = u"SignUpForm"

            # Adding extra field for password confirmation
            if adapted_schema.get_member("password"):

                password_confirmation_member = schema.String(
                    name = "password_confirmation",
                    edit_control = "cocktail.html.PasswordBox",
                    required = adapted_schema.get_member("password"),
                    after_member = "password"
                )
                adapted_schema.add_member(password_confirmation_member)

                # Add validation to compare password_confirmation and
                # password fields
                @password_confirmation_member.add_validation
                def validate_password_confirmation(member, value, ctx):
                    password = ctx.get_value("password")
                    password_confirmation = value

                    if password and password_confirmation \
                    and password != password_confirmation:
                        yield PasswordConfirmationError(member, value, ctx)

            return adapted_schema

        def submit(self):
            Form.submit(self)
            block = self.controller.block

            # Adding roles
            for role in block.roles:
                if role not in self.instance.roles:
                    self.instance.roles.append(block.roles)

            # If require email confirmation, disabled authenticated access
            confirmation_email_template = block.confirmation_email_template
            if confirmation_email_template:
                self.instance.enabled = False
                self.instance.confirmed_email = False
            else:
                self.instance.enabled = True

            self.instance.insert()

        def after_submit(self):
            block = self.controller.block

            # If require email confirmation send email confirmation message
            confirmation_email_template = block.confirmation_email_template
            if confirmation_email_template:
                confirmation_email_template.send(self.email_parameters)
                success_page = block.pending_page
            else:
                success_page = block.confirmation_page
                # Autologin
                app.authentication.set_user_session(self.instance)

            # Redirecting to the success page
            raise cherrypy.HTTPRedirect(success_page.get_uri())

        @request_property
        def email_parameters(self):
            parameters = {
                "user": self.instance,
                "confirmation_url": self.confirmation_url
            }
            return parameters

        @request_property
        def confirmation_url(self):
            uri = self.controller.block.confirmation_page.get_uri()
            location = Location.get_current(relative=False)
            location.path_info = uri
            location.query_string = {
                "email": self.instance.email,
                "hash": generate_confirmation_hash(self.instance.email)
            }
            return str(location)



#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.modeling import extend, call_base
from cocktail.translations import translations
from cocktail import schema
from cocktail.html.datadisplay import display_factory
from woost.models import Publishable

Publishable.add_member(
    schema.Reference("access_restriction",
        type = "woost.extensions.restrictedaccess.accessrestriction."
               "AccessRestriction",
        bidirectional = True,
        indexed = True,
        listed_by_default = False,
        edit_control = display_factory(
            "cocktail.html.RadioSelector",
            empty_option_displayed = True
        ),
        search_control = "cocktail.html.DropdownSelector",
        member_group = "publication"
    )
)

@extend(Publishable.access_restriction)
def translate_value(self, value, language = None, **kwargs):

    if not value:
        return translations(
            "Publishable.access_restriction=None",
            language,
            **kwargs
        )

    return call_base(value, language, **kwargs)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from bs4 import BeautifulSoup, Comment
from woost.models import get_current_website

class HTMLAdapter(object):

    def __init__(self, html):
        self.soup = BeautifulSoup(html)
        self.absolute_url_prefix = \
            "http://" + get_current_website().hosts[0]

    def adapt(self):

        # Remove all HTML comments
        is_comment = lambda text: isinstance(text, Comment)
        for comment in self.soup.findAll(text = is_comment):
            comment.extract()

        # Remove all scripts
        for element in self.soup.findAll("script"):
            element.extract()

        # Transform URLs
        for element in self.soup.findAll("a"):
            href = element.get("href")
            if href:
                element["href"] = self._transform_url(
                    href,
                    node = element
                )

        for element in self.soup.findAll("img"):
            src = element.get("src")
            if src:
                element["src"] = self._transform_url(
                    src,
                    node = element
                )

        return str(self.soup).decode("utf-8")

    def _transform_url(self, url, node = None):

        if url.startswith("mailto:"):
            return url

        # Make URLs absolute
        if "://" not in url:
            url = self.absolute_url_prefix + "/" + url.lstrip("/")

        return url




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema


class Subscriber(schema.SchemaObject):

    members_order = ["subscriber_name", "subscriber_email"]

    subscriber_name = schema.String(
        required = True
    )

    subscriber_email = schema.EmailAddress(
        required = True
    )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.html import templates
from cocktail.translations import translations
from cocktail.typemapping import TypeMapping
from woost.models import Configuration, Publishable


class ViewFactory(object):

    _type_mapping = None

    def __init__(self):
        self._type_mapping = TypeMapping()

    def _normalize_view(self, value, item, **parameters):

        if isinstance(value, basestring):
            view = templates.new(value)
            view.item = item
        elif isinstance(value, ViewFactory):
            view = value.create_view(item, **parameters)
        elif isinstance(value, type):
            view = value()
            view.item = item
        elif callable(value):
            view = value(item, parameters)

        # Set any remaining parameters as attributes of the resulting view
        if view:
            for key, value in parameters.iteritems():
                setattr(view, key, value)

        return view

    def create_view(self, item, **parameters):

        view = None

        for cls, handlers in self._type_mapping.iter_by_type(item.__class__):
            for id, handler in handlers:
                view = self._normalize_view(handler, item, **parameters)
                if view is not None:
                    return view

        raise ValueError(
            "Can't create view for item: %s" % item
        )

    def register(self, cls, id, handler):
        handlers = self._type_mapping.get(cls, recursive = False)
        if handlers is None:
            handlers = []
            self._type_mapping[cls] = handlers

        handlers.append((id, handler))

    def register_first(self, cls, id, handler):
        handlers = self._type_mapping.get(cls, recursive = False)
        if handlers is None:
            handlers = []
            self._type_mapping[cls] = handlers

        handlers.insert(0, (id, handler))

    def register_before(self, cls, anchor, id, handler):
        handlers = self._type_mapping.get(cls, recursive = False)
        if handlers is None:
            handlers = []
            self._type_mapping[cls] = handlers

        new_handlers = []
        added = False

        for prev_id, prev_handler in handlers:
            if prev_id == anchor:
                new_handlers.append((id, handler))
                added = True
            new_handlers.append((prev_id, prev_handler))

        self._type_mapping[cls] = new_handlers

        if not added:
            raise ValueError(
                "Can't register %s before %s: "
                "no such handler for type %s in %s"
                % (
                    id,
                    anchor,
                    cls,
                    self
                )
            )

    def register_after(self, cls, anchor, id, handler):
        handlers = self._type_mapping.get(cls, recursive = False)
        if handlers is None:
            handlers = []
            self._type_mapping[cls] = handlers

        new_handlers = []
        added = False

        for prev_id, prev_handler in handlers:
            new_handlers.append((prev_id, prev_handler))
            if prev_id == anchor:
                new_handlers.append((id, handler))
                added = True

        self._type_mapping[cls] = new_handlers

        if not added:
            raise ValueError(
                "Can't register %s after %s: "
                "no such handler for type %s in %s"
                % (
                    id,
                    anchor,
                    cls,
                    self
                )
            )

    def replace(self, cls, id, handler):
        handlers = self._type_mapping.get(cls, recursive = False)
        if handlers is None:
            handlers = []
            self._type_mapping[cls] = handlers

        replaced = False
        for i, handler_data in enumerate(handlers):
            prev_id, prev_handler = handler_data
            if prev_id == id:
                handlers[i] = (id, handler)
                replaced = True
                break

        if not replaced:
            raise ValueError(
                "Can't replace %s: "
                "no such handler for type %s in %s"
                % (
                    id,
                    cls,
                    self
                )
            )

    def unregister(self, cls, id):
        handlers = self._type_mapping.get(cls, recursive = False)
        if handlers is None:
            handlers = []
            self._type_mapping[cls] = handlers

        new_handlers = []

        for prev_id, prev_handler in handlers:
            if prev_id != id:
                new_handlers.append((prev_id, prev_handler))

        self._type_mapping[cls] = new_handlers


publishable_view_factory = ViewFactory()

def video_player(item, parameters):
    if item.resource_type == "video":

        player_settings = parameters.get("player_settings")

        if player_settings is None:
            player_settings_list = Configuration.instance.video_player_settings
            if player_settings_list:
                player_settings = player_settings_list[0]

        if player_settings:
            return player_settings.create_player(item)

publishable_view_factory.register(Publishable, "video_player", video_player)

def image_gallery(item, parameters):
    if item.resource_type == "image":
        view = templates.new("woost.views.ImageGallery")
        view.images = [item]
        view.labels_visible = False
        return view

publishable_view_factory.register(Publishable, "image_gallery", image_gallery)

publishable_view_factory.register(
    Publishable,
    "default_thumbnail",
    "woost.views.ThumbnailLink"
)

publishable_grid_view_factory = ViewFactory()

def video_player_dialog(item, parameters):
    if item.resource_type == "video":
        view = templates.new("woost.views.PublishablePopUp")
        view.item = item
        view.view_factory = publishable_view_factory
        return view

publishable_grid_view_factory.register(
    Publishable,
    "video_player_dialog",
    video_player_dialog
)
publishable_grid_view_factory.register(
    Publishable,
    "publishable_view_factory",
    publishable_view_factory
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations
from woost.models import Extension, Configuration, VideoPlayerSettings
from woost.models.rendering import ChainRenderer


translations.define("VimeoExtension",
    ca = u"Vimeo",
    es = u"Vimeo",
    en = u"Vimeo"
)

translations.define("VimeoExtension-plural",
    ca = u"Vimeo",
    es = u"Vimeo",
    en = u"Vimeo"
)


class VimeoExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Afegeix suport per vídeos de Vimeo""",
            "ca"
        )
        self.set("description",
            u"""Añade soporte para videos de Vimeo""",
            "es"
        )
        self.set("description",
            u"""Adds support for Vimeo videos""",
            "en"
        )

    def _load(self):
        from woost.extensions.vimeo import (
            strings,
            vimeovideo,
            videoplayersettings
        )

        self.install()

    def _install(self):
        self._create_renderers()

    def _create_renderers(self):

        from woost.extensions.vimeo.vimeovideorenderer \
            import VimeoVideoRenderer

        # Look for the first chain renderer
        for renderer in Configuration.instance.renderers:
            if isinstance(renderer, ChainRenderer):

                # Add the renderer for Vimeo videos
                vimeo_renderer = VimeoVideoRenderer()
                vimeo_renderer.insert()
                renderer.renderers.append(vimeo_renderer)

                break




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.html import Element
from mimetypes import guess_type
from woost.extensions.audio.request import get_audio_uri


class AudioPlayer(Element):

    tag = "audio"
    file = None
    encodings = [".ogg", ".mp3"]
    media_element_defaults = {}

    def _build(self):
        self["controls"] = "controls"
        self["preload"] = "auto"
        self.media_element_options = self.media_element_defaults.copy()
        self.add_resource("/cocktail/mediaelement/mediaelementplayer.min.css")
        self.add_resource("/cocktail/mediaelement/mediaelement-and-player.min.js")
        self.add_client_code("jQuery(this).mediaelementplayer(this.mediaElementOptions);")

    def _ready(self):

        self.set_client_param(
            "mediaElementOptions",
            self.media_element_options
        )

        for encoding in self.encodings:
            url = get_audio_uri(self.file, encoding)
            if url:
                source = Element("source")
                source["src"] = url

                mime_type = guess_type(url, strict = False)
                if mime_type:
                    source["type"] = mime_type[0]

                self.append(source)

        Element._ready(self)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.controllers.passwordchangecontroller import (
    PasswordChangeBlockController,
    PasswordChangeConfirmationBlockController
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from cocktail.html.datadisplay import display_factory
from woost.models import Publishable, File
from woost.models.rendering import ImageFactory
from .block import Block
from .elementtype import ElementType
from .blockimagefactoryreference import BlockImageFactoryReference

_mandatory_dropdown = display_factory(
    "cocktail.html.DropdownSelector",
    empty_option_displayed = False
)


class TextBlock(Block):

    instantiable = True
    view_class = "woost.views.TextBlockView"
    block_display = "woost.views.TextBlockDisplay"
    edit_form = "woost.views.TextBlockForm"

    groups_order = [
        "content",
        "images",
        "link",
        "behavior",
        "html",
        "administration"
    ]

    members_order = [
        "heading_alignment",
        "element_type",
        "text",
        "images",
        "image_alignment",
        "image_thumbnail_factory",
        "image_close_up_enabled",
        "image_close_up_factory",
        "image_close_up_preload",
        "image_labels_visible",
        "image_footnotes_visible",
        "image_original_link_visible",
        "link_destination",
        "link_parameters",
        "link_opens_in_new_window"
    ]

    heading_alignment = schema.String(
        required = True,
        default = "top",
        enumeration = ["top", "inside"],
        edit_control = "cocktail.html.RadioSelector",
        member_group = "content"
    )

    element_type = ElementType(
        member_group = "content"
    )

    text = schema.String(
        edit_control = "woost.views.RichTextEditor",
        translated = True,
        member_group = "content"
    )

    images = schema.Collection(
        items = schema.Reference(type = File),
        related_end = schema.Collection(),
        relation_constraints = [File.resource_type.equal("image")],
        member_group = "images",
        invalidates_cache = True
    )

    image_alignment = schema.String(
        required = True,
        default = "float_top_left",
        enumeration = [
            "float_top_left",
            "float_top_right",
            "column_left",
            "column_right",
            "top_left",
            "top_right",
            "bottom_left",
            "center_top",
            "center_bottom",
            "inline",
            "heading_icon",
            "background",
            "fallback"
        ],
        edit_control = _mandatory_dropdown,
        member_group = "images"
    )

    image_thumbnail_factory = BlockImageFactoryReference(
        required = True,
        member_group = "images"
    )

    image_close_up_enabled = schema.Boolean(
        required = True,
        default = False,
        member_group = "images"
    )

    image_close_up_factory = BlockImageFactoryReference(
        required = True,
        default = schema.DynamicDefault(
            lambda: ImageFactory.get_instance(identifier = "gallery_close_up")
        ),
        member_group = "images"
    )

    image_close_up_preload = schema.Boolean(
        required = True,
        default = True,
        member_group = "images"
    )

    image_labels_visible = schema.Boolean(
        required = True,
        default = False,
        member_group = "images"
    )

    image_footnotes_visible = schema.Boolean(
        required = True,
        default = False,
        member_group = "images"
    )

    image_original_link_visible = schema.Boolean(
        required = True,
        default = False,
        member_group = "images"
    )

    def get_block_image(self):
        for image in self.images:
            return image
        else:
            return Block.get_block_image(self)

    link_destination = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        member_group = "link",
        invalidates_cache = True
    )

    link_parameters = schema.String(
        edit_control = "cocktail.html.TextArea",
        member_group = "link"
    )

    link_opens_in_new_window = schema.Boolean(
        default = False,
        required = True,
        member_group = "link"
    )

    def init_view(self, view):
        Block.init_view(self, view)

        if self.element_type == "dd":
            view.tag = None
            view.content_wrapper.tag = "dd"
        else:
            view.tag = self.element_type

        view.text = self.text
        view.heading_alignment = self.heading_alignment
        view.images = self.images
        view.image_alignment = self.image_alignment
        view.image_thumbnail_factory = self.image_thumbnail_factory
        view.image_close_up_factory = self.image_close_up_factory
        view.image_close_up_enabled = self.image_close_up_enabled
        view.image_close_up_preload = self.image_close_up_preload
        view.image_labels_visible = self.image_labels_visible
        view.image_footnotes_visible = self.image_footnotes_visible
        view.image_original_link_visible = self.image_original_link_visible
        view.link_destination = self.link_destination
        view.link_parameters = self.link_parameters
        view.link_opens_in_new_window = self.link_opens_in_new_window

    def get_block_proxy(self, view):
        if self.element_type == "dd":
            return view.content_wrapper
        return view

    def has_heading(self):
        return self.heading or (
            self.image_alignment == "heading_icon"
            and any(image.is_accessible() for image in self.images)
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from datetime import date, datetime, timedelta
from cocktail.dateutils import add_time
from cocktail import schema
from cocktail.schema.expressions import RangeIntersectionExpression
from cocktail.controllers import (
    request_property,
    get_parameter,
    Pagination
)
from .event import Event
from .block import Block
from .elementtype import ElementType


class EventListing(Block):

    max_page_size = 50

    instantiable = True
    type_group = "blocks.listings"

    groups_order = list(Block.groups_order)
    groups_order.insert(groups_order.index("content") + 1, "listing")

    members_order = [
        "element_type",
        "include_expired",
        "listing_order",
        "paginated",
        "page_size",
        "view_class"
    ]

    element_type = ElementType(
        member_group = "content"
    )

    include_expired = schema.Boolean(
        required = True,
        default = True,
        member_group = "listing"
    )

    listing_order = schema.String(
        default = "-event_start",
        enumeration = [
            "-event_start",
            "event_start"
        ],
        member_group = "listing"
    )

    paginated = schema.Boolean(
        required = True,
        default = False,
        member_group = "listing"
    )

    page_size = schema.Integer(
        min = 1,
        required = paginated,
        member_group = "listing"
    )

    view_class = schema.String(
        required = True,
        shadows_attribute = True,
        enumeration = [
            "woost.views.CompactEventListing",
            "woost.views.DateLocationTitleEventListing"
        ],
        default = "woost.views.DateLocationTitleEventListing",
        member_group = "listing"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.tag = self.element_type
        view.name_prefix = self.name_prefix
        view.name_suffix = self.name_suffix
        view.depends_on(Event)

        if self.paginated:
            view.pagination = self.pagination
        else:
            view.events = self.select_events()

    def select_events(self):
        events = Event.select_accessible()

        if not self.include_expired:

            subset = set()

            subset.update(
                Event.select(
                    Event.event_end.greater(datetime.now())
                )
            )

            subset.update(
                Event.select(filters = [
                    Event.event_end.equal(None),
                    Event.event_start.greater(datetime.now())
                ])
            )

            events.base_collection = subset

        self._apply_listing_order(events)

        if not self.paginated and self.page_size:
            events.range = (0, self.page_size)

        return events

    def _apply_listing_order(self, events):
        if self.listing_order:
            events.order = self.listing_order

    @request_property
    def pagination(self):
        return get_parameter(
            self.pagination_member,
            errors = "set_default",
            prefix = self.name_prefix,
            suffix = self.name_suffix
        )

    @request_property
    def pagination_member(self):
        return Pagination.copy(**{
            "page_size.default": self.page_size,
            "page_size.max": self.max_page_size,
            "items": self.select_events()
        })




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from json import loads
from urllib2 import urlopen
from woost.models.rendering import ImageURIRenderer
from woost.extensions.vimeo.vimeovideo import VimeoVideo

METADATA_URL_PATTERN = "http://vimeo.com/api/v2/video/%s.json"

def get_video_metadata(video_id, timeout = 15):
    url = METADATA_URL_PATTERN % video_id
    json = urlopen(url, timeout = timeout).read()
    return loads(json)[0]


class VimeoVideoRenderer(ImageURIRenderer):

    def can_render(self, item, **parameters):
        return isinstance(item, VimeoVideo) and item.video_id

    def get_item_uri(self, item):
        return get_video_metadata(item.video_id)["thumbnail_large"]




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
import cherrypy
from createsend import CreateSend, Subscriber, BadRequest
from cocktail import schema
from cocktail.pkgutils import import_object
from cocktail.controllers import (
    Controller,
    FormProcessor,
    Form,
    request_property
)
from woost.models import Configuration


class SubscriptionController(FormProcessor, Controller):

    class SubscriptionForm(Form):

        is_new = None

        @request_property
        def model(self):
            return import_object(self.controller.block.subscriber_model)

        @request_property
        def schema(self):
            adapted_schema = Form.schema(self)

            if len(self.controller.block.lists) > 1:
                lists = schema.Collection(
                    name = "lists",
                    items = schema.Reference(
                        type = "woost.extensions.campaign3.campaignmonitorlist.CampaignMonitorList"
                    ),
                    min = 1,
                    edit_control = "cocktail.html.CheckList",
                )
                adapted_schema.add_member(lists)
                self.adapter.exclude(lists.name)

            return adapted_schema

        @request_property
        def subscribed_lists(self):
            return self.data.get("lists") or self.controller.block.lists

        @request_property
        def email(self):
            return self.data["subscriber_email"]

        @request_property
        def name(self):
            return self.data["subscriber_name"]

        @request_property
        def custom_fields(self):
            return []

        def submit(self):
            Form.submit(self)
            for list in self.subscribed_lists:
                self.add_subscriber(list)

        def add_subscriber(self, list):
            subscriber = Subscriber(
                {"api_key": Configuration.instance.get_setting("campaign_monitor_api_key")}
            )

            # Check if the user is new to any list
            try:
                response = subscriber.get(list.list_id, self.email)
            except BadRequest:
                self.is_new = True
            else:
                if response.State != "Active":
                    self.is_new = True

            response = subscriber.add(
                list.list_id,
                self.email,
                self.name,
                self.custom_fields,
                True
            )

        def after_submit(self):
            # Redirect the user to the confirmation success page of the first
            # list if the subscriber is already subscribed to all lists
            if not self.is_new:
                for list in self.subscribed_lists:
                    if list.confirmation_success_page \
                    and list.confirmation_success_page.is_accessible():
                        raise cherrypy.HTTPRedirect(list.confirmation_success_page.get_uri())

            # Redirect the user to the confirmation page
            for list in self.subscribed_lists:
                if list.confirmation_page \
                and list.confirmation_page.is_accessible():
                    raise cherrypy.HTTPRedirect(list.confirmation_page.get_uri())




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Website

Website.add_member(
    schema.String("google_analytics_account",
        text_search = False,
        member_group = "services.google_analytics",
        synchronizable = False,
        listed_by_default = False
    )
)

Website.add_member(
    schema.String("google_analytics_domain",
        text_search = False,
        member_group = "services.google_analytics",
        synchronizable = False,
        listed_by_default = False
    )
)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from warnings import warn
warn(
    "woost.controllers.authentication.AuthenticationModule is deprecated, "
    "use woost.authenticationscheme.AuthenticationScheme instead "
    "(or its global instance through woost.app.authentication)",
    DeprecationWarning
)

from woost.authenticationscheme import (
    AuthenticationScheme as AuthenticationModule,
    AuthenticationFailedError
)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2008
"""
import re
from datetime import date, datetime
from contextlib import contextmanager
from cocktail.styled import styled
from cocktail.iteration import first
from cocktail.modeling import getter, ListWrapper, SetWrapper, DictWrapper
from cocktail.events import event_handler, when, Event, EventInfo
from cocktail import schema
from cocktail.translations import translations
from cocktail.caching import whole_cache
from cocktail.caching.utils import nearest_expiration
from cocktail.persistence import (
    datastore,
    PersistentObject,
    PersistentClass,
    PersistentMapping,
    MaxValue
)
from cocktail.controllers import (
    make_uri,
    percent_encode_uri,
    resolve_object_ref,
    Location
)
from woost import app
from .websitesession import get_current_website
from .changesets import ChangeSet, Change
from .usersession import get_current_user

_protocol_regexp = re.compile(r"^[a-z][\-a-z0-9]*:")

# Extension property to determine which members should trigger a cache
# invalidation request
schema.Member.invalidates_cache = True

# Extension property to fine tune the reach of cache invalidation requests
# based on a change for a specific member
schema.Member.cache_part = None

# Extension property to denote members which affect the cache expiration time
# for its containing models
schema.Member.affects_cache_expiration = False


class Item(PersistentObject):
    """Base class for all CMS items. Provides basic functionality such as
    authorship, modification timestamps, versioning and synchronization.
    """
    type_group = "setup"
    instantiable = False

    members_order = [
        "id",
        "qname",
        "global_id",
        "synchronizable",
        "author",
        "creation_time",
        "last_update_time",
        "last_translation_update_time"
    ]

    # Enable full text indexing for all items (although the Item class itself
    # doesn't provide any searchable text field by default, its subclasses may,
    # or it may be extended; by enabling full text indexing at the root class,
    # heterogeneous queries on the whole Item class will use available
    # indexes).
    full_text_indexed = True

    # Extension property that indicates if content types should be visible from
    # the backoffice root view
    visible_from_root = True

    # Extension property that indicates if the backoffice should show child
    # entries for this content type in the type selector
    collapsed_backoffice_menu = False

    # Customization of the heading for BackOfficeItemView
    backoffice_heading_view = "woost.views.BackOfficeItemHeading"

    # Customization of the backoffice preview action
    preview_view = "woost.views.BackOfficePreviewView"
    preview_controller = "woost.controllers.backoffice." \
        "previewcontroller.PreviewController"

    def __init__(self, *args, **kwargs):
        PersistentObject.__init__(self, *args, **kwargs)

        # Assign a global ID for the object (unless one was passed in as a
        # keyword parameter)
        if self.global_id is None and self.id:
            self._generate_global_id()

    @event_handler
    def handle_inherited(cls, e):
        if (
            isinstance(e.schema, schema.SchemaClass)
            and "instantiable" not in e.schema.__dict__
        ):
            e.schema.instantiable = True

    # Unique qualified name
    #--------------------------------------------------------------------------
    qname = schema.String(
        unique = True,
        indexed = True,
        text_search = False,
        listed_by_default = False,
        member_group = "administration"
    )

    # Synchronization
    #------------------------------------------------------------------------------
    global_id = schema.String(
        required = True,
        unique = True,
        indexed = True,
        normalized_index = False,
        synchronizable = False,
        invalidates_cache = False,
        listed_by_default = False,
        member_group = "administration"
    )

    def _generate_global_id(self):

        if not app.installation_id:
            raise ValueError(
                "No value set for woost.app.installation_id; "
                "make sure your settings file specifies a unique "
                "identifier for this installation of the site."
            )

        self.global_id = "%s-%d" % (app.installation_id, self.id)

    synchronizable = schema.Boolean(
        required = True,
        indexed = True,
        synchronizable = False,
        default = True,
        shadows_attribute = True,
        invalidates_cache = False,
        listed_by_default = False,
        member_group = "administration"
    )

    # Backoffice customization
    #--------------------------------------------------------------------------
    show_detail_view = "woost.views.BackOfficeShowDetailView"
    show_detail_controller = \
        "woost.controllers.backoffice.showdetailcontroller." \
        "ShowDetailController"
    collection_view = "woost.views.BackOfficeCollectionView"
    edit_node_class = "woost.controllers.backoffice.editstack.EditNode"
    edit_view = "woost.views.BackOfficeFieldsView"
    edit_form = "woost.views.ContentForm"
    edit_controller = \
        "woost.controllers.backoffice.itemfieldscontroller." \
        "ItemFieldsController"

    __deleted = False

    @getter
    def is_deleted(self):
        return self.__deleted

    # Last change timestamp
    #--------------------------------------------------------------------------
    @classmethod
    def get_last_instance_change(cls):
        max_value = datastore.root.get(cls.full_name + ".last_instance_change")
        return None if max_value is None else max_value.value

    @classmethod
    def set_last_instance_change(cls, last_change):
        for cls in cls.__mro__:
            if hasattr(cls, "set_last_instance_change"):
                key = cls.full_name + ".last_instance_change"
                max_value = datastore.root.get(key)
                if max_value is None:
                    datastore.root[key] = max_value = MaxValue(last_change)
                else:
                    max_value.value = last_change

    # Versioning
    #--------------------------------------------------------------------------
    versioned = True

    changes = schema.Collection(
        required = True,
        versioned = False,
        editable = False,
        synchronizable = False,
        items = "woost.models.Change",
        bidirectional = True,
        invalidates_cache = False,
        visible = False,
        affects_last_update_time = False
    )

    creation_time = schema.DateTime(
        versioned = False,
        indexed = True,
        editable = False,
        synchronizable = False,
        invalidates_cache = False,
        member_group = "administration"
    )

    last_update_time = schema.DateTime(
        indexed = True,
        versioned = False,
        editable = False,
        synchronizable = False,
        invalidates_cache = False,
        affects_last_update_time = False,
        member_group = "administration"
    )

    last_translation_update_time = schema.DateTime(
        translated = True,
        indexed = True,
        versioned = False,
        editable = False,
        synchronizable = False,
        invalidates_cache = False,
        affects_last_update_time = False,
        listed_by_default = False,
        member_group = "administration"
    )

    @classmethod
    def _create_translation_schema(cls, members):
        members["versioned"] = False
        PersistentObject._create_translation_schema.im_func(cls, members)

    @classmethod
    def _add_member(cls, member):
        if member.name == "translations":
            member.editable = False
            member.searchable = False
            member.synchronizable = False
        PersistentClass._add_member(cls, member)

    def _get_revision_state(self):
        """Produces a dictionary with the values for the item's versioned
        members. The value of translated members is represented using a
        (language, translated value) mapping.

        @return: The item's current state.
        @rtype: dict
        """

        # Store the item state for the revision
        state = PersistentMapping()

        for key, member in self.__class__.members().iteritems():

            if not member.versioned:
                continue

            if member.translated:
                value = dict(
                    (language, translation.get(key))
                    for language, translation in self.translations.iteritems()
                )
            else:
                value = self.get(key)

                # Make a copy of mutable objects
                if isinstance(
                    value, (list, set, ListWrapper, SetWrapper)
                ):
                    value = list(value)

            state[key] = value

        return state

    # Item insertion overriden to make it versioning aware
    @event_handler
    def handle_inserting(cls, event):

        item = event.source
        now = datetime.now()
        item.creation_time = now
        item.last_update_time = now

        for language in item.translations:
            item.set("last_translation_update_time", now, language)

        item.set_last_instance_change(now)
        item.__deleted = False

        if item.__class__.versioned:
            changeset = ChangeSet.current

            if changeset:
                change = Change()
                change.action = "create"
                change.target = item
                change.changed_members = set(
                    member.name
                    for member in item.__class__.members().itervalues()
                    if member.versioned
                )
                change.item_state = item._get_revision_state()
                change.changeset = changeset
                changeset.changes[item.id] = change

                if item.author is None:
                    item.author = changeset.author

                change.insert(event.inserted_objects)

    # Extend item modification to make it versioning aware
    @event_handler
    def handle_changed(cls, event):

        item = event.source
        now = None

        update_timestamp = (
            item.is_inserted
            and event.member.affects_last_update_time
            and not getattr(item, "_v_is_producing_default", False)
        )

        if update_timestamp:
            now = datetime.now()
            item.set_last_instance_change(now)

        if getattr(item, "_v_initializing", False) \
        or not event.member.versioned \
        or not item.is_inserted \
        or not item.__class__.versioned:
            return

        changeset = ChangeSet.current

        if changeset:

            member_name = event.member.name
            language = event.language
            change = changeset.changes.get(item.id)

            if change is None:
                action = "modify"
                change = Change()
                change.action = action
                change.target = item
                change.changed_members = set()
                change.item_state = item._get_revision_state()
                change.changeset = changeset
                changeset.changes[item.id] = change
                if update_timestamp:
                    item.last_update_time = now
                    if event.member.translated:
                        item.set("last_translation_update_time", now, event.language)
                change.insert()
            else:
                action = change.action

            if action == "modify":
                change.changed_members.add(member_name)

            if action in ("create", "modify"):
                value = event.value

                # Make a copy of mutable objects
                if isinstance(
                    value, (list, set, ListWrapper, SetWrapper)
                ):
                    value = list(value)

                if language:
                    change.item_state[member_name][language] = value
                else:
                    change.item_state[member_name] = value
        elif update_timestamp:
            item.last_update_time = now
            if event.member.translated:
                item.set("last_translation_update_time", now, event.language)

        if (
            event.member is cls.primary_member
            and not item._v_initializing
            and item.global_id is None
        ):
            item._generate_global_id()

    @event_handler
    def handle_deleting(cls, event):

        item = event.source

        # Update the last time of modification for the item
        now = datetime.now()
        item.set_last_instance_change(now)
        item.last_update_time = now

        if item.__class__.versioned:
            changeset = ChangeSet.current

            # Add a revision for the delete operation
            if changeset:
                change = changeset.changes.get(item.id)

                if change and change.action != "delete":
                    del changeset.changes[item.id]

                if change is None \
                or change.action not in ("create", "delete"):
                    change = Change()
                    change.action = "delete"
                    change.target = item
                    change.changeset = changeset
                    changeset.changes[item.id] = change
                    change.insert()

        item.__deleted = True

    _preserved_members = frozenset([changes])

    def _should_erase_member(self, member):
        return (
            PersistentObject._should_erase_member(self, member)
            and member not in self._preserved_members
        )

    # Authorship
    #--------------------------------------------------------------------------
    author = schema.Reference(
        indexed = True,
        editable = False,
        type = "woost.models.User",
        listed_by_default = False,
        invalidates_cache = False,
        member_group = "administration"
    )

    # URLs
    #--------------------------------------------------------------------------
    def get_image_uri(self,
        image_factory = None,
        parameters = None,
        encode = True,
        include_extension = True,
        host = None,):

        uri = make_uri("/images", self.id)
        ext = None

        if image_factory:
            if isinstance(image_factory, basestring):
                pos = image_factory.rfind(".")
                if pos != -1:
                    ext = image_factory[pos + 1:]
                    image_factory = image_factory[:pos]

                from woost.models.rendering import ImageFactory
                image_factory = \
                    ImageFactory.require_instance(identifier = image_factory)

            uri = make_uri(
                uri,
                image_factory.identifier or "factory%d" % image_factory.id
            )

        if include_extension:
            from woost.models.rendering.formats import (
                formats_by_extension,
                extensions_by_format,
                default_format
            )

            if not ext and image_factory and image_factory.default_format:
                ext = extensions_by_format[image_factory.default_format]

            if not ext:
                ext = getattr(self, "file_extension", None)

            if ext:
                ext = ext.lower().lstrip(".")

            if not ext or ext not in formats_by_extension:
                ext = extensions_by_format[default_format]

            uri += "." + ext

        if parameters:
            uri = make_uri(uri, **parameters)

        return self._fix_uri(uri, host, encode)

    def _fix_uri(self, uri, host, encode):

        if encode:
            uri = percent_encode_uri(uri)

        has_protocol = _protocol_regexp.match(uri)

        if has_protocol:
            host = None

        if host:
            website = get_current_website()
            policy = website and website.https_policy

            if (
                policy == "always"
                or (
                    policy == "per_page" and (
                        getattr(self, 'requires_https', False)
                        or not get_current_user().anonymous
                    )
                )
            ):
                scheme = "https"
            else:
                scheme = "http"

            if host == ".":
                location = Location.get_current_host()
                location.scheme = scheme
                host = str(location)
            elif not "://" in host:
                host = "%s://%s" % (scheme, host)

            uri = make_uri(host, uri)

        elif not has_protocol:
            uri = make_uri("/", uri)

        return uri

    copy_excluded_members = set([
        changes,
        author,
        creation_time,
        last_update_time,
        global_id
    ])

    def get_member_copy_mode(self, member):

        from woost.models import Block
        mode = PersistentObject.get_member_copy_mode(self, member)

        if (
            mode
            and mode != schema.DEEP_COPY
            and isinstance(member, schema.RelationMember)
            and member.is_persistent_relation
            and issubclass(member.related_type, Block)
        ):
            mode = lambda block, member, value: not value.is_common_block()

        return mode

    # Caching and invalidation
    #--------------------------------------------------------------------------
    cacheable = False

    @property
    def main_cache_tag(self):
        """Obtains a cache tag that can be used to match all cache entries
        related to this item.
        """
        return "%s-%d" % (self.__class__.__name__, self.id)

    def get_cache_tags(self, language = None, cache_part = None):
        """Obtains the list of cache tags that apply to this item.

        :param language: Indicates the language for which the cache
            invalidation is being requested. If not set, the returned tags will
            match all entries related to this item, regardless of the language
            they are in.

        :param cache_part: If given, the returned tags will only match cache
            entries qualified with the specified identifier. These qualifiers
            are tipically attached by specifying the homonimous parameter of
            the `~woost.views.depends_on` extension method.
        """
        main_tag = self.main_cache_tag

        if cache_part:
            main_tag += "-" + cache_part

        tags = set([main_tag])

        if language:
            tags.add("lang-" + language)

        return tags

    def get_cache_expiration(self):
        expiration = None

        for member in self.__class__.iter_members():
            if member.affects_cache_expiration:
                expiration = nearest_expiration(expiration, self.get(member))

        return expiration

    @classmethod
    def get_cache_expiration_for_type(cls):
        expiration = None

        for member in cls.iter_members():
            if member.affects_cache_expiration:

                if isinstance(member, schema.Date):
                    threshold = date.today()
                else:
                    threshold = datetime.now()

                instance = first(
                    cls.select(
                        member.greater(threshold),
                        order = member,
                        cached = False
                    )
                )

                if instance is not None:
                    expiration = nearest_expiration(
                        expiration,
                        instance.get(member)
                    )

        return expiration

    def clear_cache(self, language = None, cache_part = None):
        """Remove all the cached pages that are based on this item.

        :param language: Indicates the language for which the cache
            invalidation is being requested. If not set, the invalidation will
            affect all entries related to this item, regardless of the language
            they are in.

        :param cache_part: If given, only cache entries qualified with the
            specified identifier will be cleared. These qualifiers are
            tipically attached by specifying the homonimous parameter of the
            `~woost.views.depends_on` extension method.
        """
        app.cache.clear(
            scope = self.get_cache_invalidation_scope(
                language = language,
                cache_part = cache_part
            )
        )

    def clear_cache_after_commit(self, language = None, cache_part = None):
        """Remove all the cached pages that are based on this item, as soon as
        the current database transaction is committed.

        This method can be called multiple times during a single transaction.
        All the resulting invalidation targets will be removed from the cache
        once the transaction is committed.

        :param language: Indicates the language for which the cache
            invalidation is being requested. If not set, the invalidation will
            affect all entries related to this item, regardless of the language
            they are in.

        :param cache_part: If given, only cache entries qualified with the
            specified identifier will be cleared. These qualifiers are
            tipically attached by specifying the homonimous parameter of the
            `~woost.views.depends_on` extension method.
        """
        app.cache.clear_after_commit(
            scope = self.get_cache_invalidation_scope(
                language = language,
                cache_part = cache_part
            )
        )

    def get_cache_invalidation_scope(self, language = None, cache_part = None):
        """Determine the scope of a cache invalidation request for this item.

        :param language: Indicates the language for which the cache
            invalidation is being requested. If not set, the scope will include
            all entries related to this item, regardless of the language they
            are in.

        :param cache_part: If given, only cache entries qualified with the
            specified identifier will be included. These qualifiers are
            tipically attached by specifying the homonimous parameter of the
            `~woost.views.depends_on` extension method.

        :return: A cache invalidation scope. See the
            `cocktail.caching.cache.Cache.clear` method for details on its
            format.
        """
        selectors = set()

        if language:
            languages = list(self.iter_derived_translations(
                language,
                include_self = True
            ))

        # Tags per type
        for cls in \
        self.__class__.ascend_inheritance(include_self = True):
            selector = cls.full_name

            if cache_part:
                selector += "-" + cache_part

            if language:
                for lang in languages:
                    selectors.add((selector, "lang-" + lang))
            else:
                selectors.add(selector)

            if cls is Item:
                break

        # Tags per instance
        selector = self.main_cache_tag

        if cache_part:
            selector += "-" + cache_part

        if language:
            for lang in languages:
                selectors.add((selector, "lang-" + lang))
        else:
            selectors.add(selector)

        return selectors


Item.id.versioned = False
Item.id.editable = False
Item.id.synchronizable = False
Item.id.listed_by_default = False
Item.id.member_group = "administration"
Item.changes.visible = False

@resolve_object_ref.implementation_for(Item)
def resolve_item_ref(cls, ref):
    try:
        ref = int(ref)
    except ValueError:
        return cls.get_instance(global_id = ref)
    else:
        return cls.get_instance(ref)

# Trigger cache invalidation when items are altered
@when(Item.inserted)
def _clear_cache_after_insertion(e):
    if app.cache.enabled:
        e.source.clear_cache_after_commit()

@when(Item.deleting)
def _clear_cache_on_deletion(e):
    if app.cache.enabled:
        e.source.clear_cache_after_commit()

@when(Item.changed)
def _clear_cache_after_change(e):
    if (
        app.cache.enabled
        and e.member.invalidates_cache
        and e.source.is_inserted
    ):
        e.source.clear_cache_after_commit(
            language = e.language,
            cache_part = e.member.cache_part
        )

@when(Item.adding_translation)
@when(Item.removing_translation)
def _clear_cache_after_change(e):
    if app.cache.enabled and e.source.is_inserted:
        e.source.clear_cache_after_commit()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from simplejson import dumps
import cherrypy
from cocktail.translations import translations
from woost.models import Configuration
from woost.controllers.filecontroller import FileController
from woost.controllers.uricontroller import URIController
from woost.extensions.googleanalytics import GoogleAnalyticsExtension


class GAEventRedirection(object):

    delay = 300

    overridable_category = True
    overridable_action = True
    overridable_label = True

    default_category = None
    default_action = "open"

    template = """
        <!DOCTYPE html>
        <html>
            <head>
                <meta http-equiv="Content-Type" content="text/html;charset=utf-8">
                <title>%(title)s</title>
                %(analytics_script)s
                <script type="text/javascript">
                    window.onload = function () {
                        setTimeout(function () {
                            location.replace(%(url)s);
                        }, %(delay)d);
                    }
                </script>
            </head>
            <body>
                %(body)s
            </body>
        </html>
        """

    def __init__(self, controller_class):

        base_impl = controller_class.__call__

        def __call__(controller, **kwargs):

            publishable = controller.context["publishable"]

            if self.event_tracking_is_enabled(controller, publishable):

                cherrypy.response.headers["Content-Type"] = "text/html"

                url = self.get_url(controller, publishable)
                category = self.get_category(controller, publishable)
                action = self.get_action(controller, publishable)
                label = self.get_label(controller, publishable)

                api = Configuration.instance.google_analytics_api

                if api == "ga.js":
                    commands = [('_trackEvent', category, action, label)]
                elif api == "universal":
                    commands = [('send', 'event', category, action, label)]
                else:
                    raise cherrypy.HTTPRedirect(url)

                return self.template % {
                    "title": self.get_title(controller, publishable),
                    "analytics_script":
                        GoogleAnalyticsExtension.instance.get_analytics_script(
                            publishable = publishable,
                            async = False,
                            commands = commands
                        ),
                    "url": dumps(url),
                    "delay": self.delay,
                    "body": self.get_body(controller, publishable, url = url)
                }

            # Google Analytics event tracking disabled
            else:
                return base_impl(controller, **kwargs)

        controller_class.__call__ = __call__
        controller_class.ga_event_redirection = self

    def event_tracking_is_enabled(self, controller, publishable):
        return cherrypy.request.params.get("ga_event") == "1"

    def get_url(self, controller, publishable):

        extra_params = dict(
            (key, value)
            for key, value in cherrypy.request.params.iteritems()
            if not key.startswith("ga_event")
        )

        return publishable.get_uri(parameters = extra_params)

    def get_title(self, controller, publishable):
        return translations("woost.extensions.googleanalytics.redirection_title")

    def get_body(self, controller, publishable, url):
        return translations(
            "woost.extensions.googleanalytics.redirection_body",
            url = url
        )

    def get_category(self, controller, publishable):
        return (
            self.overridable_category
            and cherrypy.request.params.get("ga_event_category")
            or self.default_category
            or publishable.__class__.__name__
        )

    def get_action(self, controller, publishable):
        return (
            self.overridable_action
            and cherrypy.request.params.get("ga_event_action")
            or self.default_action
        )

    def get_label(self, controller, publishable):
        return (
            self.overridable_label
            and cherrypy.request.params.get("ga_event_label")
            or str(publishable.id)
        )

GAEventRedirection(FileController)
GAEventRedirection(URIController)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from woost.models import Configuration
from woost.extensions.blocks.block import Block
from woost.extensions.blocks.elementtype import ElementType
from woost.extensions.issuu.issuudocument import IssuuDocument


class IssuuBlock(Block):

    instantiable = True
    view_class = "woost.extensions.issuu.IssuuBlockView"
    block_display = "woost.extensions.issuu.IssuuBlockDisplay"

    members_order = ["element_type", "issuu_document", "width", "height"]

    element_type = ElementType(
        member_group = "content"
    )

    issuu_document = schema.Reference(
        type = IssuuDocument,
        required = True,
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "content"
    )

    width = schema.Integer(
        listed_by_default = False,
        member_group = "content"
    )

    height = schema.Integer(
        listed_by_default = False,
        member_group = "content"
    )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from warnings import warn
warn(
    "woost.controllers.module.Module is deprecated, and its subclasses "
    "(LanguageModule and AuthenticationModule) have been relocated to "
    "woost.app",
    DeprecationWarning
)


class Module(object):

    def __init__(self, application):
        self.application = application




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from cocktail.controllers import (
    request_property,
    get_parameter,
    Pagination
)
from .block import Block
from .elementtype import ElementType
from .news import News


class NewsListing(Block):

    max_page_size = 50

    instantiable = True
    type_group = "blocks.listings"

    groups_order = list(Block.groups_order)
    groups_order.insert(groups_order.index("content") + 1, "listing")

    members_order = [
        "element_type",
        "paginated",
        "page_size",
        "view_class"
    ]

    element_type = ElementType(
        member_group = "content"
    )

    paginated = schema.Boolean(
        required = True,
        default = False,
        member_group = "listing"
    )

    page_size = schema.Integer(
        min = 1,
        required = paginated,
        member_group = "listing"
    )

    view_class = schema.String(
        required = True,
        shadows_attribute = True,
        enumeration = [
            "woost.views.CompactNewsListing",
            "woost.views.TextOnlyNewsListing",
            "woost.views.TextAndImageNewsListing"
        ],
        default = "woost.views.TextOnlyNewsListing",
        member_group = "listing"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.tag = self.element_type
        view.name_prefix = self.name_prefix
        view.name_suffix = self.name_suffix
        view.depends_on(News)

        if self.paginated:
            view.pagination = self.pagination
        else:
            view.news = self.select_news()

    def select_news(self):
        news = News.select_accessible(order = "-news_date")

        if not self.paginated and self.page_size:
            news.range = (0, self.page_size)

        return news

    @request_property
    def pagination(self):
        return get_parameter(
            self.pagination_member,
            errors = "set_default",
            prefix = self.name_prefix,
            suffix = self.name_suffix
        )

    @request_property
    def pagination_member(self):
        return Pagination.copy(**{
            "page_size.default": self.page_size,
            "page_size.max": self.max_page_size,
            "items": self.select_news()
        })




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from time import time, sleep
from subprocess import Popen, PIPE
from cocktail import schema
from woost.models.item import Item
from woost.extensions.audio.audiodecoder import AudioDecoder


class AudioEncoder(Item):

    instantiable = True
    visible_from_root = False
    resolution = 0.25

    members_order = [
        "identifier",
        "mime_type",
        "extension",
        "command",
        "timeout",
        "timeout_size_factor"
    ]

    identifier = schema.String(
        required = True,
        unique = True,
        indexed = True,
        normalized_index = False,
        descriptive = True
    )

    mime_type = schema.String(
        required = True
    )

    extension = schema.String(
        required = True
    )

    command = schema.String(
        required = True
    )

    timeout = schema.Integer(
        required = True,
        default = 60
    )

    timeout_size_factor = schema.Float(
        default = 10.0
    )

    def encode(self, file, dest):

        # Most encoders expect PCM wave files as input, so we start by finding
        # an appropiate decoder for the given file.
        mime_type = file.mime_type

        if mime_type == "audio/mp3":
            mime_type = "audio/mpeg"

        decoder = AudioDecoder.require_instance(mime_type = mime_type)

        # Next, we produce the command line instructions for the decoding /
        # encoding procedure, using a shell pipe.
        decode_command = decoder.command % file.file_path
        encode_command = self.command % dest
        command = decode_command + " | " + encode_command

        # Calculate the timeout, based on a base value incremented according to
        # a file size factor
        timeout = self.timeout

        if self.timeout_size_factor:
            size = file.file_size
            if size:
                timeout += size / (self.timeout_size_factor * 1024 * 1024)

        # Encode the file and save it to the indicated location
        proc = Popen(command, shell = True, stdout = PIPE, stderr = PIPE)
        start = time()

        while proc.poll() is None:
            if time() - start > timeout:
                proc.terminate()
                raise AudioEncodingTimeoutError(
                    "The following audio encoding command exceeded its "
                    "timeout of %d seconds: %s" % (timeout, command)
                )
            sleep(self.resolution)

        if proc.returncode:
            raise AudioEncodingCommandError(
                "The following audio encoding command reported an error: "
                "%s\n%s" % (command, proc.stderr.read().strip())
            )


class AudioEncodingTimeoutError(IOError):
    """An exception raised when an audio encoding request exceeds its
    timeout.
    """

class AudioEncodingCommandError(OSError):
    """An exception raised when the underlying command used by an audio encoder
    reports an error.
    """




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.modeling import abstractmethod
from cocktail import schema
from woost.models.item import Item


class Renderer(Item):

    instantiable = False
    visible_from_root = False

    title = schema.String(
        descriptive = True,
        translated = True
    )

    @abstractmethod
    def can_render(self, item, **parameters):
        """Indicates if the renderer is able to render the given item.

        @param item: The item to evaluate.
        @type item: L{Item<woost.models.item.Item>}

        @return: True if the renderer claims to be able to render the item,
            False otherwise.
        @rtype: bool
        """

    @abstractmethod
    def render(self, item, **parameters):
        """Produces an image for the given item.

        @param item: The item to render.
        @type item: L{Item<woost.models.item.Item>}

        @return: The image for the given item. If a suitable image file
            exists, the method should return a tuple consisting of a string
            pointing to its path and its MIME type. Otherwise, the method
            should craft the image in-memory and return it as an instance of
            the L{Image<Image.Image>} class.
        @rtype: tuple(str, str) or L{Image<Image.Image>}
        """




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

translations.define("woost.extensions.annotations.Annotation-instance",
    ca = lambda instance, **kwargs:
        translations(instance.annotation_target, **kwargs) + u" (notes)",
    es = lambda instance, **kwargs:
        translations(instance.annotation_target, **kwargs) + u" (notas)",
    en = lambda instance, **kwargs:
        translations(instance.annotation_target, **kwargs) + u" (notes)"
)

translations.define(
    "woost.extensions.annotations.FormOverlay.annotationDropdown.label",
    ca = u"Annotacions per aquest camp",
    es = u"Anotaciones para este campo",
    en = u"Annotations for this field"
)

